from .datastorage import DataStorage


class Filesystem(DataStorage):

    # overriding abstract method
    def load_dataset(self, spark, read_all=None, input_dest='', header=True):

        if input_dest == '':
            input_dataset = self.args["input-dataset"]
        else:
            input_dataset = input_dest
        delimiter = self.args["delimiter"] if "delimiter" in self.args else ","
        input_file_format = self.args["inputFileFormat"] if "inputFileFormat" in self.args else "csv"

        return spark.read.load(input_dataset, format=input_file_format, sep=delimiter, inferSchema=True, header=header, error_bad_lines=False)

    # overriding abstract method
    def save_dataset(self, df, output_dest):

        if output_dest == '':
            output_dataset = self.args["output-dataset"]
        else:
            output_dataset = output_dest
        delimiter = self.args["delimiter"] if "delimiter" in self.args else ","
        output_file_format = self.args["outputFileFormat"] if "outputFileFormat" in self.args else "csv"

        if "save_dataset_with_pandas" in self.args:

            df.toPandas().to_csv(output_dataset, sep=delimiter, index=False)

        else:

            df.write.options(delimiter=delimiter).\
                mode('overwrite').\
                option("header", True).\
                save(output_dataset, format=output_file_format)



from .datastorage import DataStorage


class Presto(DataStorage):

    # Initialize Presto Python Driver

    # overriding abstract method
    def load_dataset(self, spark, read_all=True, input_dest='', header=True):

        input_columns = self.args["input-columns"] if "input-columns" in self.args else "*"

        if input_dest == '':
            input_dataset = self.args["input-dataset"]
        else:
            input_dataset = input_dest

        # To manage supervised algorithms
        if "labelCol" in self.args:
            label_col = self.args["labelCol"]
            columns_list = input_columns.split(",")
            if label_col not in columns_list:
                columns_list.append(label_col)
            input_columns = ','.join(columns_list)

        if "prestoPassword-input-dataset" in self.args and self.args["prestoPassword-input-dataset"] not in ['', "", "''"]:

            if read_all:
                df = spark.read.format("jdbc").option("driver", "com.facebook.presto.jdbc.PrestoDriver"). \
                    option("url", "jdbc:presto://"+self.args["prestoHost-input-dataset"]+":"+self.args["prestoPort-input-dataset"]+"/"+self.args["prestoDb-input-dataset"]).option("dbtable", "(SELECT * FROM " + input_dataset + ")"). \
                    option("user", self.args["prestoUser-input-dataset"]). \
                    option('password', self.args["prestoPassword-input-dataset"]).load()
            else:
                df = spark.read.format("jdbc").option("driver", "com.facebook.presto.jdbc.PrestoDriver"). \
                    option("url", "jdbc:presto://"+self.args["prestoHost-input-dataset"]+":"+self.args["prestoPort-input-dataset"]+"/"+self.args["prestoDb-input-dataset"]).option("dbtable", "(SELECT " + input_columns + " FROM " + input_dataset + ")"). \
                    option("user", self.args["prestoUser-input-dataset"]). \
                    option('password', self.args["prestoPassword-input-dataset"]).load()

        else:
            if read_all:
                df = spark.read.format("jdbc").option("driver", "com.facebook.presto.jdbc.PrestoDriver"). \
                    option("url", "jdbc:presto://"+self.args["prestoHost-input-dataset"]+":"+self.args["prestoPort-input-dataset"]+"/"+self.args["prestoDb-input-dataset"]).option("dbtable", "(SELECT * FROM " + input_dataset + ")"). \
                    option("user", self.args["prestoUser-input-dataset"]).load()
            else:
                df = spark.read.format("jdbc").option("driver", "com.facebook.presto.jdbc.PrestoDriver"). \
                    option("url", "jdbc:presto://"+self.args["prestoHost-input-dataset"]+":"+self.args["prestoPort-input-dataset"]+"/"+self.args["prestoDb-input-dataset"]).option("dbtable", "(SELECT " + input_columns + " FROM " + input_dataset + ")"). \
                    option("user", self.args["prestoUser-input-dataset"]).load()

        # rename column
        for column in input_columns.split(","):
            df = df.withColumnRenamed(column.lower(), column)
        return df

    # overriding abstract method
    def save_dataset(self, df, output_dest):
        pass



from .datastorage import DataStorage


class Mongo(DataStorage):

    def __init__(self, args):
        super().__init__(args)
        user = self.args['mongoUser']
        password = self.args['mongoPassword']
        ip = self.args['mongoUri'].split(":")[0] 
        port = self.args['mongoUri'].split(":")[1] 
        self.uri = "mongodb://" + user + ":" + password + "@" + ip + ":" + port + "/"


    # overriding abstract method
    def load_dataset(self, spark, read_all=True, input_dest=None, header=True):

        if input_dest is None:
            input_dest = self.args["input-dataset"]
   
        db = input_dest.split["."][0]
        collection = input_dest.split["."][1]

        df = spark.read.format("com.mongodb.spark.sql.DefaultSource")\
            .option("uri", self.uri).option("database", db).option("collection", collection)\
            .load()
        return df


    # overriding abstract method
    def save_dataset(self, df, output_dest=None):

        if output_dest is None:
            output_dest = self.args["output-dataset"]
       
        db = output_dest.split["."][0]
        collection = output_dest.split["."][1]

        df.write.format("com.mongodb.spark.sql.DefaultSource").mode("append")\
            .option("uri", self.uri).option("database", db).option("collection", collection)\
            .save()



from pyspark.sql import SparkSession
from pyspark.sql import HiveContext


def spark_initializer(app_name, args):

    input_datastorage_type = ""
    output_datastorage_type = ""

    if "dataStorageType-input-dataset" in args:
        input_datastorage_type = args["dataStorageType-input-dataset"].lower()

    if "dataStorageType-output-dataset" in args:
        output_datastorage_type = args["dataStorageType-output-dataset"].lower()

    if input_datastorage_type.startswith("hive"):
        spark = SparkSession \
            .builder \
            .appName(app_name) \
            .config('spark.sql.warehouse.dir', '/warehouse/tablespace/managed/hive') \
            .config("spark.sql.legacy.allowCreatingManagedTableUsingNonemptyLocation", "true") \
            .config('hive.metastore.uris', args["hiveMetastoreUris-input-dataset"]) \
            .config('spark.hadoop.dfs.client.use.datanode.hostname', True) \
            .config('javax.jdo.option.ConnectionUserName', args["hiveUserName-input-dataset"]) \
            .config('javax.jdo.option.ConnectionPassword', args["hivePassword-input-dataset"]) \
            .config('hive.server2.enable.doAs', True) \
            .config('hive.metastore.client.connect.retry.delay', 5) \
            .config('hive.metastore.client.socket.timeout', 1800) \
            .config('spark.jars.packages', 'org.mongodb.spark:mongo-spark-connector_2.12:2.4') \
            .enableHiveSupport() \
            .getOrCreate()
    elif output_datastorage_type.startswith("hive"):
        spark = SparkSession \
            .builder \
            .appName(app_name) \
            .config('spark.sql.warehouse.dir', '/warehouse/tablespace/managed/hive') \
            .config("spark.sql.legacy.allowCreatingManagedTableUsingNonemptyLocation", "true") \
            .config('hive.metastore.uris', args["hiveMetastoreUris-output-dataset"]) \
            .config('spark.hadoop.dfs.client.use.datanode.hostname', True) \
            .config('javax.jdo.option.ConnectionUserName', args["hiveUserName-output-dataset"]) \
            .config('javax.jdo.option.ConnectionPassword', args["hivePassword-output-dataset"]) \
            .config('hive.server2.enable.doAs', True) \
            .config('hive.metastore.client.connect.retry.delay', 5) \
            .config('hive.metastore.client.socket.timeout', 1800) \
            .config('spark.jars.packages', 'org.mongodb.spark:mongo-spark-connector_2.12:2.4') \
            .enableHiveSupport() \
            .getOrCreate()

        # Hive session
        hive = HiveContext(spark)
    else:
        spark = SparkSession.builder.appName(app_name).getOrCreate()

    return spark
